from __future__ import annotations

import os
from pathlib import Path
from typing import List, NamedTuple, Optional, Tuple, Union

import mecab_ko_dic

import _mecab

mecabrc_path = Path(__file__).absolute().parent / "mecabrc"


def _create_lattice(sentence: str) -> _mecab.Lattice:
    lattice = _mecab.Lattice()
    lattice.add_request_type(_mecab.MECAB_ALLOCATE_SENTENCE)  # Required
    lattice.set_sentence(sentence)

    return lattice


class Feature(NamedTuple):
    pos: str
    semantic: Optional[str] = None
    has_jongseong: Optional[bool] = None
    reading: Optional[str] = None
    type: Optional[str] = None
    start_pos: Optional[str] = None
    end_pos: Optional[str] = None
    exprssion: Optional[str] = None

    @classmethod
    def _from_node(cls, node: _mecab.Node) -> Feature:
        # Reference:
        # - http://taku910.github.io/mecab/learn.html
        # - https://docs.google.com/spreadsheets/d/1-9blXKjtjeKZqsf4NzHeYJCrr49-nXeRF6D80udfcwY
        # - https://bitbucket.org/eunjeon/mecab-ko-dic/src/master/utils/dictionary/lexicon.py

        # feature = <pos>,<semantic>,<has_jongseong>,<reading>,<type>,<start_pos>,<end_pos>,<expression>
        values = node.feature.split(",")
        assert len(values) == 8

        feature = {
            field: value if value != "*" else None
            for field, value in zip(Feature._fields, values)
        }

        if feature["has_jongseong"] == "T":
            feature["has_jongseong"] = True
        elif feature["has_jongseong"] == "F":
            feature["has_jongseong"] = False

        return cls(**feature)

    def __str__(self) -> str:
        feature = {
            key: value if value is not None else "*"
            for key, value in self._asdict().items()
        }
        # True -> T / False -> F / * -> *
        feature["has_jongseong"] = str(feature["has_jongseong"])[0]
        return ",".join(feature.values())


class MeCabError(Exception):
    pass


class MeCab:  # APIs are inspired by KoNLPy
    def __init__(
        self,
        dictionary_path: Optional[os.PathLike] = None,
        user_dictionay_path: Optional[Union[os.PathLike, List[os.PathLike]]] = None,
    ):
        if dictionary_path is None:
            dictionary_path = str(mecab_ko_dic.dictionary_path)

        user_dictionay_argument = []
        if user_dictionay_path is not None:
            if not isinstance(user_dictionay_path, list):
                user_dictionay_path = [user_dictionay_path]

            user_dictionay_argument = [
                "--userdic",
                ",".join(f'"{path}"' for path in user_dictionay_path),
            ]

        arguments = [
            "--rcfile",
            str(mecabrc_path),
            "--dicdir",
            dictionary_path,
            *user_dictionay_argument,
        ]
        self._tagger = _mecab.Tagger(arguments)

    def parse(self, sentence: str) -> List[Tuple[str, Feature]]:
        lattice = _create_lattice(sentence)
        if not self._tagger.parse(lattice):
            raise MeCabError(self._tagger.what())

        return [(node.surface, Feature._from_node(node)) for node in lattice]

    def pos(self, sentence: str) -> List[Tuple[str, str]]:
        return [(surface, feature.pos) for surface, feature in self.parse(sentence)]

    def morphs(self, sentence: str) -> List[str]:
        return [surface for surface, _ in self.parse(sentence)]

    def nouns(self, sentence: str) -> List[str]:
        return [
            surface
            for surface, feature in self.parse(sentence)
            if feature.pos.startswith("N")
        ]



import sys
from typing import List, Optional, Tuple

import mecab_ko_dic

import _mecab
from mecab import mecabrc_path

rcfile_argument = ["--rcfile", str(mecabrc_path)]
dicdir_argument = ["--dicdir", str(mecab_ko_dic.dictionary_path)]
model_argument = ["--model", str(mecab_ko_dic.model_path)]

def parse_arguments() -> Tuple[Optional[str], List[str]]:
    arguments = sys.argv[1:]
    if not arguments:
        return None, []

    tasks = {"dict-index", "dict-gen", "cost-train"}
    if arguments[0] in tasks:
        return arguments[0], arguments[1:]

    return None, arguments


def mecab_dict_index(arguments: List[str]) -> int:
    return _mecab.cli.dict_index([*dicdir_argument, *model_argument, *arguments])


def mecab_dict_gen(arguments: List[str]) -> int:
    return _mecab.cli.dict_gen([*dicdir_argument, *model_argument, *arguments])


def mecab_cost_train(arguments: List[str]) -> int:
    return _mecab.cli.cost_train([*dicdir_argument, *arguments])


def mecab(arguments: List[str]) -> int:
    return _mecab.cli.mecab([*rcfile_argument, *dicdir_argument, *arguments])


def main() -> int:
    task, arguments = parse_arguments()

    if task == "dict-index":
        return mecab_dict_index(arguments)
    elif task == "dict-gen":
        return mecab_dict_gen(arguments)
    elif task == "cost-train":
        return mecab_cost_train(arguments)
    else:
        return mecab(arguments)


if __name__ == "__main__":
    sys.exit(main())



import os
import platform
import shutil
import site
import subprocess
import sys
from glob import glob
from pathlib import Path
from typing import Optional

from pybind11.setup_helpers import Pybind11Extension
from pybind11.setup_helpers import build_ext as _build_ext
from setuptools import setup

is_windows = (platform.system() == "Windows")

prefix_paths = [
    Path(sys.prefix),
    Path(site.getuserbase()),
    Path.home() / ".local",
]


class Executable:
    def __init__(self, command: str):
        self._command = command

    def __call__(self, *args) -> str:
        return subprocess.check_output(
            [self.executable, *args], encoding="utf-8"
        ).strip()

    def exists(self) -> bool:
        return self.executable is not None

    @property
    def executable(self) -> Optional[str]:
        paths = [str(path / "bin") for path in prefix_paths]
        paths.append(os.environ["PATH"])
        return shutil.which(self._command, path=os.pathsep.join(paths))


class unix_build_ext(_build_ext):
    def build_extension(self, extension):
        if extension.name == "_mecab":
            mecab_config = Executable("mecab-config")
            if not mecab_config.exists():
                sys.stderr.write("==================================================\n")
                sys.stderr.write("You need mecab-ko to build the extension.\n")
                sys.stderr.write("Please install mecab-ko as follows:\n")
                sys.stderr.write(
                    "$ wget https://raw.githubusercontent.com/jonghwanhyeon/python-mecab-ko/main/scripts/install_mecab_ko.py\n"
                )
                sys.stderr.write("$ python3 install_mecab_ko.py\n")
                sys.stderr.write("==================================================\n")
                raise RuntimeError("mecab-ko not found")

            extension.include_dirs.append(mecab_config("--inc-dir"))
            extension.library_dirs.append(mecab_config("--libs-only-L"))
            extension.libraries.append("mecab")

        super().build_extension(extension)


class windows_ext(_build_ext):
    def build_extension(self, extension):
        if extension.name == "_mecab":
            extension.include_dirs.append(r"C:\mecab")
            extension.library_dirs.append(r"C:\mecab")
            extension.libraries.append("libmecab")

        super().build_extension(extension)


setup(
    cmdclass={
        "build_ext": unix_build_ext if not is_windows else windows_ext
    },
    ext_modules=[
        Pybind11Extension(
            name="_mecab",
            sources=sorted(glob("mecab/pybind/**/*.cpp", recursive=True)),
            include_dirs=["mecab/pybind/_mecab"],
        ),
    ],
)